import { EntityEquippableComponent, EntityRideableComponent, EquipmentSlot, MinecraftDimensionTypes, Player, system, world } from "@minecraft/server";
const sounds = [
    {
        id: "stevelines.flint_and_steel",
        events: {
            itemUseOn: {
                item: "minecraft:flint_and_steel"
            }
        }
    },
    {
        id: "stevelines.chicken_jockey",
        events: {
            findMob: {
                mob: "minecraft:chicken",
                radius: 16,
                hasPassenger: true
            }
        }
    },
    {
        id: "stevelines.iron_golem",
        events: {
            findMob: {
                mob: "minecraft:iron_golem",
                radius: 6
            }
        }
    },
    {
        id: "stevelines.midport_village",
        events: {
            findMob: {
                mob: "minecraft:villager",
                radius: 48
            }
        }
    },
    {
        id: "stevelines.the_villagers",
        events: {
            findMob: {
                mob: "minecraft:villager",
                radius: 8
            }
        }
    },
    {
        id: "stevelines.firework_rocket",
        events: {
            itemUse: {
                item: "minecraft:firework_rocket"
            }
        }
    },
    {
        id: "stevelines.enderpearl",
        events: {
            itemUse: {
                item: "minecraft:ender_pearl"
            }
        }
    },
    {
        id: "stevelines.elytra",
        events: {
            equipItems: [{
                    item: "minecraft:elytra",
                    slots: [EquipmentSlot.Chest]
                }]
        }
    },
    {
        id: "stevelines.diamond_armor",
        events: {
            equipItems: [
                {
                    item: "minecraft:diamond_helmet",
                    slots: [EquipmentSlot.Head]
                },
                {
                    item: "minecraft:diamond_chestplate",
                    slots: [EquipmentSlot.Chest]
                },
                {
                    item: "minecraft:diamond_leggings",
                    slots: [EquipmentSlot.Legs]
                },
                {
                    item: "minecraft:diamond_boots",
                    slots: [EquipmentSlot.Feet]
                }
            ]
        }
    },
    {
        id: "stevelines.release",
        events: {
            itemUseOn: {
                item: "minecraft:water_bucket"
            }
        }
    },
    {
        id: "stevelines.the_nether",
        events: {
            enterDimension: {
                dimension: MinecraftDimensionTypes.nether
            }
        }
    },
    {
        id: "stevelines.crafting_table",
        events: {
            placeBlock: {
                block: "minecraft:crafting_table"
            }
        }
    },
    {
        id: "stevelines.tnt",
        events: {
            placeBlock: {
                block: "minecraft:tnt"
            }
        }
    },
    {
        id: "stevelines.skadoosh",
        events: {
            killMob: {
                mob: "minecraft:ender_dragon"
            }
        }
    },
    {
        id: "stevelines.lava_chicken",
        events: {
            itemCompleteUse: {
                item: "minecraft:cooked_chicken"
            }
        }
    },
    {
        id: "stevelines.bread",
        events: {
            itemCompleteUse: {
                item: "minecraft:bread"
            }
        }
    },
    {
        id: "stevelines.i_am_steve",
        events: {
            random: {
                interval: 1200,
                chance: 5
            }
        }
    },
    {
        id: "stevelines.coming_in_hot",
        events: {
            onFall: {
                maxVelocity: -0.75
            }
        }
    }
];
function playSoundFromPlayer(player, sound) {
    player.playSound(`${sound}.ui`);
    player.dimension.getEntities({ type: "player", location: player.location, maxDistance: 16 }).forEach((otherPlayer) => {
        if (otherPlayer.id === player.id)
            return;
        otherPlayer.playSound(sound, { location: player.location });
    });
}
world.afterEvents.entityDie.subscribe((data) => {
    if (!data.deadEntity || !data.deadEntity.isValid())
        return;
    const player = data.damageSource.damagingEntity;
    if (!(player instanceof Player))
        return;
    for (const sound of sounds) {
        if (!sound.events.killMob)
            continue;
        if (sound.events.killMob.mob && (data.deadEntity.typeId !== sound.events.killMob.mob))
            continue;
        playSoundFromPlayer(player, sound.id);
    }
});
world.afterEvents.itemUseOn.subscribe((data) => {
    for (const sound of sounds) {
        if (!sound.events.itemUseOn)
            continue;
        if (data.itemStack.typeId !== sound.events.itemUseOn.item)
            continue;
        playSoundFromPlayer(data.source, sound.id);
    }
});
world.afterEvents.itemUse.subscribe((data) => {
    for (const sound of sounds) {
        if (!sound.events.itemUse)
            continue;
        if (data.itemStack.typeId !== sound.events.itemUse.item)
            continue;
        playSoundFromPlayer(data.source, sound.id);
    }
});
world.afterEvents.itemCompleteUse.subscribe((data) => {
    for (const sound of sounds) {
        if (!sound.events.itemCompleteUse)
            continue;
        if (data.itemStack.typeId !== sound.events.itemCompleteUse.item)
            continue;
        playSoundFromPlayer(data.source, sound.id);
    }
});
world.afterEvents.playerPlaceBlock.subscribe((data) => {
    for (const sound of sounds) {
        if (!sound.events.placeBlock)
            continue;
        if (data.block.typeId !== sound.events.placeBlock.block)
            continue;
        playSoundFromPlayer(data.player, sound.id);
    }
});
const foundMob = {};
system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        function* job(sound) {
            if (!sound.events.findMob)
                return;
            let hasMob = false;
            player.dimension.getEntities({ type: sound.events.findMob.mob, location: player.location, maxDistance: sound.events.findMob.radius }).forEach((entity) => {
                if (sound.events.findMob?.hasPassenger && !entity.getComponent(EntityRideableComponent.componentId)?.getRiders()[0])
                    return;
                hasMob = true;
            });
            if (!hasMob) {
                if (foundMob[`${player.id}.${sound.id}`]) {
                    player.dimension.getEntities({ type: sound.events.findMob.mob, location: player.location, maxDistance: sound.events.findMob.radius * 1.5 }).forEach((entity) => {
                        if (sound.events.findMob?.hasPassenger && !entity.getComponent(EntityRideableComponent.componentId)?.getRiders()[0])
                            return;
                        hasMob = true;
                    });
                }
                if (hasMob)
                    return;
                if (foundMob[`${player.id}.${sound.id}`])
                    delete foundMob[`${player.id}.${sound.id}`];
                return;
            }
            if (foundMob[`${player.id}.${sound.id}`])
                return;
            playSoundFromPlayer(player, sound.id);
            foundMob[`${player.id}.${sound.id}`] = true;
        }
        for (const sound of sounds)
            system.runJob(job(sound));
    }
}, 20);
const equippedItems = {};
system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        const slots = [];
        const comp = player.getComponent(EntityEquippableComponent.componentId);
        for (const slot in EquipmentSlot) {
            const item = comp.getEquipment(slot);
            slots.push({ slot: slot, item: item?.typeId });
        }
        for (const sound of sounds) {
            if (!sound.events.equipItems)
                continue;
            let hasAllItems = true;
            for (const item of sound.events.equipItems) {
                let hasItem = false;
                for (const slot of slots) {
                    if (slot.item === item.item && item.slots.includes(slot.slot))
                        hasItem = true;
                }
                if (!hasItem)
                    hasAllItems = false;
            }
            if (!hasAllItems) {
                if (equippedItems[`${player.id}.${sound.id}`])
                    delete equippedItems[`${player.id}.${sound.id}`];
                continue;
            }
            if (equippedItems[`${player.id}.${sound.id}`])
                continue;
            playSoundFromPlayer(player, sound.id);
            equippedItems[`${player.id}.${sound.id}`] = true;
        }
    }
}, 5);
world.afterEvents.playerDimensionChange.subscribe((data) => {
    if (!data.player || !data.player.isValid())
        return;
    const interval = system.runInterval(() => {
        if (!data.player || !data.player.isValid())
            return;
        const vel = data.player.getVelocity();
        if (vel.x !== 0 || vel.y !== 0 || vel.z !== 0)
            return;
        for (const sound of sounds) {
            if (!sound.events.enterDimension)
                continue;
            if (sound.events.enterDimension.dimension !== data.toDimension.id)
                continue;
            playSoundFromPlayer(data.player, sound.id);
        }
        system.clearRun(interval);
    }, 3);
});
for (const sound of sounds) {
    if (!sound.events.random)
        continue;
    system.runInterval(() => {
        for (const player of world.getAllPlayers()) {
            if (sound.events.random?.chance && (Math.random() * 100) > sound.events.random.chance)
                continue;
            playSoundFromPlayer(player, sound.id);
        }
    }, sound.events.random.interval);
}
const fallingPlayers = {};
system.runInterval(() => {
    for (const player of world.getAllPlayers()) {
        const velocity = player.getVelocity();
        for (const sound of sounds) {
            if (!sound.events.onFall)
                continue;
            if (velocity.y > sound.events.onFall.maxVelocity || player.isGliding || player.isOnGround || player.isSwimming) {
                if (fallingPlayers[`${player.id}.${sound.id}`])
                    delete fallingPlayers[`${player.id}.${sound.id}`];
                continue;
            }
            if (fallingPlayers[`${player.id}.${sound.id}`])
                continue;
            playSoundFromPlayer(player, sound.id);
            fallingPlayers[`${player.id}.${sound.id}`] = true;
        }
    }
}, 5);
